
/*
 * Copyright (c) Developers of JMST
 * Redistributable and modifiable. Please see license.readme for details.
 *
 * ammendment history
 * <date>			<changed by>		<summary of changes>
 * 25 August 2004	Jonee Ryan Ty		First documented history
 *
 */

import java.awt.Point;
import java.util.Vector;
import java.util.Enumeration;


public class Edge
{
	class SameNodeException		extends RuntimeException {}
	class NotANodeException		extends RuntimeException {}
	
	public GNode node1, node2;						// vertices of this line
	
	private static int RADIUS = Constants.RADIUS;
	
	public boolean edgIsIncluded;

	/**
	 * Edge constructor. 
	 * @param node1, node2 vertices of this edge
	 * @exception SameNodeException when node1 == node2
	 */ 	
	public Edge(GNode node1, GNode node2)	throws SameNodeException { 			
		if(node1 == node2)
			throw new SameNodeException();
			
		this.node1 = node1;
		this.node2 = node2;		
		
		edgIsIncluded = false;
	}
			
	/**
	 * Tests whether this edge has param vertex as a vertex.
	 * @param vertex the node we wish to determine wether incident 
	 *	to this edge
	 * @return true if param is a vertex of this line false otherwise
	 */
	public boolean isIncidentTo(GNode vertex) {
		return ((node1 == vertex) || (node2 == vertex));
	}
	
	/**
	 * Tests wether this edge is incident to any node in a collection.
	 * @param vertices the collection we wish to determine wether 
	 * this edge is incident to any member of
	 * @return the node that is incident to the edge
	 */
	
	public GNode isIncidentToAny(Vector vertices) {	
		GNode nodeThatIsIncident = null, tmp;
	
		Enumeration enumNodes = vertices.elements();
			
		while(enumNodes.hasMoreElements()) {
			tmp = ((GNode)enumNodes.nextElement());
			
			if(this.isIncidentTo(tmp))		// if tmp is a vertex of this line
				nodeThatIsIncident = tmp;
		}
		
		return nodeThatIsIncident;
	}	
	
	/** Returns the other vertex of this edge.
	 *@exception NotANodeException when the param is not a node of this edge
	 *@param nde the node whose "partner" we wish to find out
	 *@return the other node in this edge
	 */
	 
	public GNode otherNode(GNode nde)		throws NotANodeException {
		if(nde == node1) 
			return node2;
		
		else if(nde == node2)
			return node1;	
			
		else
			throw new NotANodeException();	
	}
	
	/** 
	 * Returns the endpoints of this edge.
	 * @return a point array where (p[0].x, p[0].y) and (p[1].x, p[1].y) 
	 * is the endpoints of this edge
	 */
	
	public Point[] getBounds() {	
		Point[] p = new Point[2];
		
		p[0] = new Point(node1.x + RADIUS, node1.y + RADIUS);
		p[1] = new Point(node2.x + RADIUS, node2.y + RADIUS);
		
		return p;
	}
	
	/** 
	 * Returns the pixel length of this edge.
	 * @return pixel length of this edge
	 */
	
	public int getWeight() {
		return (int)Math.round(Math.sqrt(Math.pow((node1.x - node2.x), 2) + 
			Math.pow((node1.y - node2.y), 2)));
	}	
		
} //##30
