
/*
 * Copyright (c) Developers of JMST
 * Redistributable and modifiable. Please see license.readme for details.
 *
 * ammendment history
 * <date>			<changed by>		<summary of changes>
 * 25 August 2004	Jonee Ryan Ty		First documented history
 *
 */

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Graphics;

import java.awt.event.MouseMotionListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

import java.util.Vector;
import java.util.Enumeration;

 
public class GCanv		extends javax.swing.JPanel
						implements MouseListener, MouseMotionListener
{
	public static final int 	UNWEIGHTED_ORDERED 	= 0,	//graph types
								WEIGHTED_UNORDERED	= 1;
								
	public int graphType = 1, colorCounter = 0;	
	
	public boolean 	mouseEventsAreBlocked = false,
					showCopyright = true,
					needToRepaint = true;
					
	public Graph graph;
										
	private static final int	LEFT_CLICK = 16, // mouse event modifiers
								RIGHT_CLICK = 4;	
	
	private GNode drawingFromNode, selectedNode;	
	private Edge deleted;
	private Point drawingToPoint;
			
	private boolean	isDrawingALine = false,
					deletedALine = false,
					addedselectedNode = false;
		// index of selected node in v gNodes of graph
	private int nodeIndex = -1;

		// mouse coordinates and allowances
	private int xMouse, yMouse, xExtra, yExtra;			 
	
	public GCanv() {		
		super();
			// default graph type is GCanv.UNWEIGHTED_ORDERED		
		this.setBackground(Constants.BACK_COL);
		
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		
		graph = new Graph();
			
		this.setPreferredSize(
			new Dimension(Constants.CANV_WIDTH, Constants.CANV_HEIGHT));
	}
	
	public GCanv(int graphType) {
		this();	
		this.graphType = graphType;
	}
	
	public void mouseClicked(MouseEvent e) {}
	
	public void mouseEntered(MouseEvent e) {}
   
	public void mouseExited(MouseEvent e) {}
	
	public void mousePressed(MouseEvent e) {		
		if (mouseEventsAreBlocked)
			return;
			
		xMouse = e.getX();
     	yMouse = e.getY();
     		
     	nodeIndex = -1;					// rank in v nodes of a clicked node
     	deletedALine = false;	
     		
		if(e.getModifiers() == LEFT_CLICK) {
			Enumeration enum = graph.gNodes.elements();	
														
			GNode gTemp;
				// go through all nodes to see if mouse is on top of any
			while(enum.hasMoreElements()) {	
				gTemp = (GNode)enum.nextElement();
			
				if(this.inCircle(gTemp.x, gTemp.y, xMouse, yMouse)) {
					if(e.getClickCount() != 2) {	// not double clicked
						xExtra = xMouse - gTemp.x;
     					yExtra = yMouse - gTemp.y;	
					}
							// last index if nodes overlaps
					nodeIndex = graph.gNodes.indexOf(gTemp);	
					selectedNode = gTemp;
				}											
			}
							// double clicking on a line deletes it
			if((e.getClickCount() == 2) && (nodeIndex == -1)) {			
				enum = graph.gEdges.elements();			// reuse variable
				Edge tmp;
							
				while(enum.hasMoreElements()) {			// go through all lines
					tmp = (Edge)enum.nextElement();
					
					if(this.edgeIsClicked(tmp.getBounds(), xMouse, yMouse)) {
						graph.removeEdge(tmp);
						deletedALine = true;
						deleted = tmp;
						graph.gEdges.trimToSize();	
						System.gc();
					}
				}					
			}				
						// double clicking on empty space creates a new node
			if((e.getClickCount() == 2) &&(nodeIndex == -1)&&(!deletedALine)) {
				selectedNode = new GNode(xMouse - Constants.RADIUS, 
					yMouse - Constants.RADIUS,
					Constants.COL[colorCounter++ % Constants.COL.length]);
				graph.addNode(selectedNode);
				addedselectedNode = true;
			}	 
			// end of if(e.getModifiers() == LEFT_CLICK)		
		} else if((e.getModifiers() == RIGHT_CLICK) &&(e.getClickCount()==1)) {
				// user is about to draw a line from a node
												
			Enumeration enum = graph.gNodes.elements();	
			GNode temp;
				// to find drawingFromNode	
			while(enum.hasMoreElements()) {	
				temp = (GNode)enum.nextElement();
			
				if(this.inCircle(temp.x, temp.y, xMouse, yMouse)) {
					drawingFromNode = temp;		// last node if nodes overlap
					isDrawingALine = true;
				}											
			}
		}	
	} // end of mousePressed()
	
	public void mouseDragged(MouseEvent e) {
		if (mouseEventsAreBlocked)
			return;
		
		if((nodeIndex != -1) && (e.getModifiers() == LEFT_CLICK)) {	
			// if a node was dragged
			GNode dragged = selectedNode;
			
			dragged.x = e.getX() - xExtra;
			dragged.y = e.getY() - yExtra;			
		}
					// user is drawing a line
		else if(isDrawingALine)
			drawingToPoint = new Point(e.getX(), e.getY());
		
		repaint();
	}
	
	public void repaintEdgeArea(int x1, int y1, int x2, int y2) {
		int x, y;
		
		x = Math.min(x1, x2);
		y = Math.min(y1, y2);
								
		repaint(x - 2 * Constants.DIAMETER, y - 2 * Constants.DIAMETER, 
			Math.abs(x1 - x2) + 4 * Constants.DIAMETER,
			Math.abs(y1 - y2) + 4 * Constants.DIAMETER);					
	}
	
	public void repaintNodeArea(GNode node) {
		repaint(node.x, node.y, Constants.DIAMETER + 1, Constants.DIAMETER + 1);
	}
	
	public void mouseReleased(MouseEvent e) {
		if (mouseEventsAreBlocked)
			return;		
		
		if (needToRepaint) {
			repaint();
			needToRepaint = false;
		}
		
		int xMo = e.getX(),
			yMo = e.getY();
									// to delete node if double clicked
		if((nodeIndex != -1) && (e.getModifiers() == LEFT_CLICK) 
				&& (e.getClickCount() == 2)) {
			GNode rem = selectedNode;
			
			graph.removeNode(rem);
			repaint();	// the node may have edges attached to it
					
		} else if(isDrawingALine)	{	// check if the user connects two nodes
			isDrawingALine = false;	
			
			Enumeration enum = graph.gNodes.elements();
			GNode temp, node2 = null;
			
			while(enum.hasMoreElements()) {	
				temp = (GNode)enum.nextElement();
						// last node if nodes overlap
				if(this.inCircle(temp.x, temp.y, xMo, yMo))		
					node2 = temp;
			}
					// if the user connected nodes, node2 is not null
			if(node2 != null) { 							
				graph.addEdge(drawingFromNode, node2);
				
				repaintEdgeArea(drawingFromNode.x, drawingFromNode.y, 
					node2.x, node2.y);
			
			} else   // if not repaint the area of the line that was released
				repaintEdgeArea(xMo, yMo, drawingFromNode.x, drawingFromNode.y);
		
		} else if (addedselectedNode) {
			addedselectedNode = false;
			repaintNodeArea(selectedNode);
			
		} else if (deletedALine) {
			deletedALine = false;
			Point[] p = deleted.getBounds();
			
			repaintEdgeArea(p[0].x, p[0].y, p[1].x, p[1].y);
		}
		
	} // end of mouseReleased	
	
	public void mouseMoved(MouseEvent e) {}
	
	public boolean inCircle(int xCir, int yCir, int xMo, int yMo) {	
   		int x = xMo - (xCir + Constants.RADIUS),
   			y = yMo - (yCir + Constants.RADIUS);
   		
   		return (x * x + y * y <= Constants.RADIUS * Constants.RADIUS);
	}					
    	
	public boolean edgeIsClicked(Point[] bounds, int xMo, int yMo) {
		if(bounds[0].x != bounds[1].x) {		// slope is not undefine
			double m = (double)(bounds[1].y - bounds[0].y) / 
				(bounds[1].x - bounds[0].x);			
				// b = 1, a = -m, point to line distance
			double c = m * bounds[0].x - bounds[0].y;
			
			return ((Math.abs(-m * xMo + yMo + c) / Math.sqrt(1 + m * m)) <= 
				Constants.ALLOWANCE);
		
		} else  									// slope is undefine
			return ((yMo < Math.max(bounds[0].y, bounds[1].y)) && 
				(yMo > Math.min(bounds[0].y, bounds[1].y)) && 
				((xMo - bounds[0].x) <= Constants.ALLOWANCE)); 

	}
	
	/**
	 * @param p is the bounds of the edge (ie p = edge.getBounds())
	 */
	public void drawEdge(Graphics g, Point[] p) {
		g.setColor(Constants.LINE_COLOR);
		g.drawLine(p[0].x, p[0].y, p[1].x, p[1].y);
	}
	
	public void drawEdge(Graphics g, int x1, int y1, int x2, int y2) {
		g.setColor(Constants.LINE_COLOR);
		g.drawLine(x1, y1, x2, y2);
	}
	
	public void drawNode(Graphics g, GNode node) {
		g.setColor(node.nodeColor);
		g.fillOval(node.x, node.y, Constants.DIAMETER, Constants.DIAMETER);
	}
	
	/**
	 * would write in the middle of the parameter line
	 */
	public void writeEdge(Graphics g, Point[] p, String toWrite) {
		g.setColor(Constants.LINE_WRITE);
		g.drawString(toWrite, (p[0].x + p[1].x) / 2 - toWrite.length() * 3,
			(p[0].y + p[1].y) / 2 + 5);			
	}
	
	public void writeNode(Graphics g, GNode node, String toWrite) {
		g.setColor(Constants.NODE_WRITE);		
		g.drawString(toWrite, node.x - toWrite.length() * 3 + Constants.RADIUS,
			node.y + Constants.DIAMETER); 		
	}
		
	public void paintComponent(Graphics g) {	
	
		super.paintComponent(g);
		
		Enumeration enum = graph.gEdges.elements(); 		
		Edge tmpEdg;
		Point p[];
		String toWrite;
		int num = 0;	
		
		while(enum.hasMoreElements()) {				// to draw all edges
			tmpEdg = (Edge)enum.nextElement();
			p = tmpEdg.getBounds();
			
			drawEdge(g, p);
		
			if (graphType == GCanv.UNWEIGHTED_ORDERED) {
				toWrite = "e" + ++num;		
				writeEdge(g, p, toWrite);
			}
			
			if (graphType == GCanv.WEIGHTED_UNORDERED) {
													// get weight
				toWrite = Integer.toString((int)Math.round(Math.sqrt(
					Math.pow((p[0].x - p[1].x), 2) + 
					Math.pow((p[0].y - p[1].y), 2))));			
													// weight centered at edge
				writeEdge(g, p, toWrite);
			}
		}
		
		if(isDrawingALine)
			drawEdge(g, drawingFromNode.x + Constants.RADIUS, 
				drawingFromNode.y + Constants.RADIUS,
				drawingToPoint.x, drawingToPoint.y);
		
		enum = graph.gNodes.elements();				// reuse variable
		GNode tempNde;
		num = 0;
		
		while(enum.hasMoreElements()) {				// to draw all nodes
			tempNde = (GNode)enum.nextElement();
			
			drawNode(g, tempNde);
			
			if (graphType == GCanv.UNWEIGHTED_ORDERED) {
				toWrite = "n" + ++num;
				writeNode(g, tempNde, toWrite);
			}
		}
		
		paintOthers(g);
		
		if (showCopyright) {
			showCopyright = false;
			g.drawString(Constants.COPYRIGHT, 10, 15);
		}			
		
	} // end of paint()
		// do algorithm specific painting in here
	public void paintOthers(Graphics g) {}
						
} //##30	
