
/*
 * Copyright (c) Developers of JMST
 * Redistributable and modifiable. Please see license.readme for details.
 *
 * ammendment history
 * <date>			<changed by>		<summary of changes>
 * 25 August 2004	Jonee Ryan Ty		First documented history
 *
 */

import java.awt.Graphics;
import java.awt.Point;
import java.awt.Color;
import javax.swing.JOptionPane;

import java.util.Enumeration;
import java.util.Vector;


public class MSTCanv		extends GCanv	
{
	public boolean isSolving = false;
	
	public Vector tmpVNodes, tmpVEdges;
	public GNode random;
	
	public String errorMsg;
	public int algo;
	
	public static final int 	PRIM	= 0,
								KRUSKAL	= 1;
	
	private boolean newEdgCouldBeFound;
	private AnimateMST anim;

	public MSTCanv() {
		super(GCanv.WEIGHTED_UNORDERED);
		
		tmpVEdges = new Vector();
		tmpVNodes = new Vector();
	}
		
	public void paintOthers(Graphics g) {
		Enumeration enum;
		Edge tmpEdg;
		Point p[];
		String toWrite;
		
		if(isSolving) {
			g.setColor(Color.red);
			
			enum = tmpVEdges.elements(); 		
			
			while(enum.hasMoreElements()) {			
					// to draw all edges that are included in the mst so far
				tmpEdg = (Edge)enum.nextElement();		// reused variables
				p = tmpEdg.getBounds();
			
				g.drawLine(p[0].x, p[0].y, p[1].x, p[1].y);
			}
				
			g.setColor(Color.blue);
			
			enum = tmpVNodes.elements();
			GNode tempNde;
			
			while(enum.hasMoreElements()) {
					// to draw all nodes that are included in the 
				tempNde = (GNode)enum.nextElement();				
				g.fillOval(tempNde.x, tempNde.y, Constants.DIAMETER, 
					Constants.DIAMETER);	
			}
		}				
	}
	
	/**
	 * "Solves" the mst problem using Prim's algorithm. Call animate() 
	 * afterwards to see the solution. what this function basically does is to 
	 * add the edges (of the mst solution) into tmpVEdges. 
	 *	 
	 * @return false if the graph is not connected, true otherwise
	 */	
	public boolean doPrim() {
	
		if(graph.gNodes.size() == 0) {			// check if a "graph" exists
			errorMsg = Constants.ERROR_3;
			return false;
		}	
		
		algo = MSTCanv.PRIM;
		
		boolean graphIsConnected = true;
			// minimal check for connectedness: check if a node is isolated
		for(int b = 0; b < graph.gNodes.size(); b++)
			if(((GNode)graph.gNodes.elementAt(b)).nodes.size() == 0) {
				graphIsConnected = false;
				b = graph.gNodes.size() + 1;
			}
		
		if(!graphIsConnected) {
			errorMsg = Constants.ERROR_1;
			return false;								
		}	
						
		tmpVEdges.removeAllElements();			// clean slate
		tmpVNodes.removeAllElements();

		newEdgCouldBeFound = true;
			// arrange edges in order of increasing weights
		graph.arrangeEdges();			
		graph.unincludeAll();			// clean slate		
													// choose any node
		random = (GNode)graph.gNodes.elementAt(
			(int)(((long)(Math.random() * Long.MAX_VALUE)) % 
			graph.gNodes.size()));

		tmpVNodes.add(random);
			
		Edge tmpEdg;	
		GNode incidentNode, otherNode;
		
		int numOfEdgesNeeded = graph.gNodes.size() - 1, edgesCounted = 0;
		
		while(newEdgCouldBeFound) { 				
			newEdgCouldBeFound = false;
		// find edge with minimum weight that connects	
		// any included point but not yet included		
			if(edgesCounted < numOfEdgesNeeded) {		
				for(int a = 0; a < graph.gEdges.size(); a++) {
					tmpEdg = (Edge)graph.gEdges.elementAt(a);
			// the edge should not have been included
			// and is incident to any node included
					if(!tmpEdg.edgIsIncluded) {		
						incidentNode = tmpEdg.isIncidentToAny(tmpVNodes);
				
						if(incidentNode != null) {	
							otherNode = tmpEdg.otherNode(incidentNode);
							
							if(!tmpVNodes.contains(otherNode)) {	
								// it should not form a cycle
						
								tmpVEdges.add(tmpEdg);
								tmpEdg.edgIsIncluded = true;
					
								tmpVNodes.add(tmpEdg.otherNode(incidentNode));
									// terminate loop
								a = graph.gEdges.size() + 1;		
								newEdgCouldBeFound = true;
								edgesCounted++;
							}
						}
					}
				}
			}
		}
		
		if (tmpVNodes.size() != graph.gNodes.size()) {
			errorMsg = Constants.ERROR_2;
			return false;						
		}
		
		tmpVEdges.trimToSize();
		tmpVNodes.trimToSize();
		
		System.gc();
		return true;
	}
	
	/**
	 * "Solves" the mst problem using Kruskal's algorithm. Call animate() 
	 * afterwards to see the solution. what this function basically does is to 
	 * add the edges (of the mst solution) into tmpVEdges. 
	 *	 
	 * @return false if the graph is not connected, true otherwise
	 */		
	public boolean doKruskal() {
	
		if(graph.gNodes.size() == 0) {			// check if a "graph" exists
			errorMsg = Constants.ERROR_3;
			return false;
		}	
		
		algo = MSTCanv.KRUSKAL;
		
		boolean graphIsConnected = true;
			// minimal check for connectedness: check if a node is isolated
		for(int b = 0; b < graph.gNodes.size(); b++)
			if(((GNode)graph.gNodes.elementAt(b)).nodes.size() == 0) {
				graphIsConnected = false;
				b = graph.gNodes.size() + 1;
			}
		
		if(!graphIsConnected) {
			errorMsg = Constants.ERROR_1;
			return false;								
		}	
						
		tmpVEdges.removeAllElements();		// clean slate
		tmpVNodes.removeAllElements();
		
		graph.arrangeEdges();			
		graph.unincludeAll();				// clean slate		
				
		Edge tmpEdge;
		GNode node1, node2;

		for (int i = 0; i < graph.gEdges.size(); i++) {
		
			tmpEdge = (Edge)graph.gEdges.elementAt(i);
			node1 = tmpEdge.node1;
			node2 = tmpEdge.node2;
			
			if (tmpVEdges.size() == graph.gNodes.size() - 1) {
				i = graph.gEdges.size();	// to terminate loop
				continue;
			}
						
			if (node1.whereIncluded == null && node2.whereIncluded == null) {

				Tree tmpTree = new Tree();
				tmpTree.addEdge(tmpEdge);	
				
				tmpVNodes.add(node1);			
				tmpVNodes.add(node2);
			
			} else if (node1.whereIncluded == null && 
					node2.whereIncluded != null) {
						
				node2.whereIncluded.addNode(node1);
				tmpVNodes.add(node1);				
			
			} else if (node1.whereIncluded != null && 
					node2.whereIncluded == null) {

				node1.whereIncluded.addNode(node2);
				tmpVNodes.add(node2);
			
			} else if (node1.whereIncluded != node2.whereIncluded) {
				Tree merge = 
					Tree.mergeTree(node1.whereIncluded, node2.whereIncluded);
				merge.gEdges.add(tmpEdge);

			} else if (node1.whereIncluded == node2.whereIncluded)
				continue;	// would create a cycle, so discard the edge
				
			tmpVEdges.add(tmpEdge);
		}

		if (tmpVEdges.size() != graph.gNodes.size() - 1) {
			errorMsg = Constants.ERROR_2;
			return false;						
		}

		tmpVEdges.trimToSize();
		tmpVNodes.trimToSize();
		
		System.gc();
		return true;
	}
	
	/**
	 * this method is called after calling either doPrim or doKruskal and it 
	 * would animate the solution of one algorithm.
	 */
	public void animate() {		
		needToRepaint = true;
		anim = new AnimateMST(this, Constants.ANIM);		
		anim.start();		
	}
	
	public void destroy() {		// to destroy any existing thread
		if (anim != null) {
			try {
				if (anim.isAlive()) {
						// longer delay so it can be interrupted
					anim.delay = 1000;	
					anim.interrupt();
					anim.canv = null;
					anim = null;
				}
			
			} catch (Exception ex) {
				ex.printStackTrace();
				System.out.println("caught\n");				
			}
		
		}
	}
	
} //##30
